@@ -466,15 +466,20 @@ int rtas_errinjct_close(int otoken)
  */
 int rtas_errinjct_open(int *otoken)
 {
-	__be32 be_otoken;
+	__be32 be_status;
 	int rc, status;
 
 	rc = sanity_check();
 	if (rc)
 		return rc;
 
-	rc = rtas_call("ibm,open-errinjct", 0, 2, &be_otoken, &status);
-	*otoken = be32toh(be_otoken);
+	/*
+	 * Unlike other RTAS calls, here first output parameter is otoken,
+	 * not status. rtas_call converts otoken to host endianess. We
+	 * have to convert status parameter.
+	 */
+	rc = rtas_call("ibm,open-errinjct", 0, 2, otoken, &be_status);
+	status = be32toh(be_status);
 
 	dbg("(%p) = %d, %d\n", otoken, rc ? rc : status, *otoken);
 	return rc ? rc : status;
@@ -508,7 +513,7 @@ int rtas_get_config_addr_info2(uint32_t config_addr, uint64_t phb_id,
 
 	*info = be32toh(be_info);
 
-	dbg("(0x%x, 0x%lx, %d) = %d, 0x%x\n", config_addr, phb_id, func,
+	dbg("(0x%x, 0x%"PRIx64", %d) = %d, 0x%x\n", config_addr, phb_id, func,
 	    rc ? rc : status, *info);
 	return rc ? rc : status;
 }
@@ -810,7 +815,7 @@ int rtas_get_vpd(char *loc_code, char *workarea, size_t size,
 
 /**
  * rtas_lpar_perftools
- * @brief Interface to the ibm,lpa-perftools rtas call
+ * @brief Interface to the ibm,lpar-perftools rtas call
  *
  * @param subfunc
  * @param workarea additional args to the rtas call
@@ -881,7 +886,7 @@ int rtas_platform_dump(uint64_t dump_tag, uint64_t sequence, void *buffer,
 	uint32_t next_hi, next_lo;
 	uint32_t bytes_hi, bytes_lo;
 	uint32_t dump_tag_hi, dump_tag_lo;
-	void *kernbuf;
+	void *kernbuf = NULL;
 	int rc, status;
 
 	rc = sanity_check();
@@ -930,7 +935,7 @@ int rtas_platform_dump(uint64_t dump_tag, uint64_t sequence, void *buffer,
 	bytes_lo = be32toh(bytes_lo);
 	*bytes_ret = BITS64(bytes_hi, bytes_lo);
 
-	dbg("(0x%lx, 0x%lx, %p, %zd, %p, %p) = %d, 0x%lx, 0x%lx\n",
+	dbg("(0x%"PRIx64", 0x%"PRIx64", %p, %zd, %p, %p) = %d, 0x%"PRIx64", 0x%"PRIx64"\n",
 	     dump_tag, sequence, buffer, length, seq_next, bytes_ret,
 	     rc ? rc : status, *seq_next, *bytes_ret);
 	return rc ? rc : status;
@@ -962,7 +967,7 @@ int rtas_read_slot_reset(uint32_t cfg_addr, uint64_t phbid, int *state,
 	*state = be32toh(*state);
 	*eeh = be32toh(*eeh);
 
-	dbg("(0x%x, 0x%lx, %p, %p) = %d, %d, %d\n", cfg_addr, phbid, state,
+	dbg("(0x%x, 0x%"PRIx64", %p, %p) = %d, %d, %d\n", cfg_addr, phbid, state,
 	     eeh, rc ? rc : status, *state, *eeh);
 	return rc ? rc : status;
 }
@@ -1074,7 +1079,7 @@ int rtas_set_eeh_option(uint32_t cfg_addr, uint64_t phbid, int function)
 		       htobe32(BITS32_HI(phbid)), htobe32(BITS32_LO(phbid)),
 		       htobe32(function), &status);
 
-	dbg("(0x%x, 0x%lx, %d) = %d\n", cfg_addr, phbid, function,
+	dbg("(0x%x, 0x%"PRIx64", %d) = %d\n", cfg_addr, phbid, function,
 	     rc ? rc : status);
 	return rc ? rc : status;
 }
@@ -1324,3 +1329,66 @@ int rtas_update_properties(char *workarea, unsigned int scope)
 	dbg("(%p) %d = %d\n", workarea, scope, rc ? rc : status);
 	return rc ? rc : status;
 }
+
+/**
+ * rtas_physical_attestation
+ * @brief Interface for ibm,physical-attestation rtas call.
+ *
+ * @param workarea input/output work area for rtas call
+ * @param seq_num sequence number of the rtas call
+ * @param next_seq_num next sequence number
+ * @param work_area_bytes size of work area
+ * @return 0 on success, !0 on failure
+ */
+int rtas_physical_attestation(char *workarea, int seq_num, int *next_seq_num,
+			      int *work_area_bytes)
+{
+	uint32_t workarea_pa;
+	uint64_t elapsed = 0;
+	void *kernbuf;
+	int kbuf_sz = 4096;
+	int rc, status;
+	int resp_bytes = *work_area_bytes;
+
+	rc = sanity_check();
+	if (rc)
+		return rc;
+
+	/* Caller provided more data than FW can handle */
+	if (*work_area_bytes == 0 ||
+	    *work_area_bytes > kbuf_sz)
+		return RTAS_IO_ASSERT;
+
+	rc = rtas_get_rmo_buffer(kbuf_sz, &kernbuf, &workarea_pa);
+	if (rc)
+		return rc;
+	memcpy(kernbuf, workarea, *work_area_bytes);
+
+	do {
+		rc = rtas_call("ibm,physical-attestation", 3, 3,
+			       htobe32(workarea_pa), htobe32(kbuf_sz),
+			       htobe32(seq_num),
+			       &status, next_seq_num, &resp_bytes);
+		if (rc < 0)
+			break;
+
+		rc = handle_delay(status, &elapsed);
+	} while (rc == CALL_AGAIN);
+
+	*next_seq_num = be32toh(*next_seq_num);
+
+	/* FW returned more data than we can handle */
+	if (be32toh(resp_bytes) > *work_area_bytes) {
+		(void)rtas_free_rmo_buffer(kernbuf, workarea_pa, kbuf_sz);
+		return RTAS_IO_ASSERT;
+	}
+
+	*work_area_bytes = be32toh(resp_bytes);
+
+	if (rc == 0)
+		memcpy(workarea, kernbuf, *work_area_bytes);
+
+	(void)rtas_free_rmo_buffer(kernbuf, workarea_pa, kbuf_sz);
+
+	return rc ? rc : status;
+}
